# Generated by Django 5.1.6 on 2025-03-01 17:24

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='VSServer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('vs_server_name', models.CharField(max_length=512)),
                ('address', models.CharField(max_length=256)),
                ('port', models.PositiveIntegerField()),
                ('play_style_id', models.CharField(max_length=64)),
                ('play_style_lang_code', models.CharField(max_length=64)),
                ('is_whitelisted', models.BooleanField()),
                ('description', models.TextField()),
                ('max_players', models.SmallIntegerField()),
                ('game_version', models.CharField(max_length=64)),
                ('has_password', models.BooleanField()),
                ('server_url', models.CharField(max_length=256, null=True)),
                ('last_heartbeat', models.DateTimeField(auto_now_add=True)),
                ('players', models.SmallIntegerField(default=0)),
                ('token', models.CharField(max_length=64)),
            ],
        ),
        migrations.CreateModel(
            name='VSMod',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('_id', models.CharField(max_length=64)),
                ('version', models.CharField(max_length=64)),
                ('server', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='mods', to='vintagestory_masterserver.vsserver')),
            ],
        ),
    ]
